local Candidato = require("Candidato")
local Coligacao = require("Coligacao")
local Partido = require("Partido")

local Urna = {}

function Urna:new (sigla)
  self.__index = self

  return setmetatable({
      candidatos = {},
      partidos = {},
      coligacoes = {},
      numeroVagas = 0,
      totalVotos = 0
      }, self)
end

function Urna:hasColigacao(coligacao)
  for i,x in pairs(self.coligacoes) do
    if x.partidos == coligacao then return x end
  end

  return nil
end

function Urna:hasPartido(partido)
  for i,x in pairs(self.partidos) do
    if x.sigla == partido then return x end
  end

  return nil
end


function Urna:addPartido(coluna, Candidato)
  local siglas = split(coluna, "-")

  local partido = self:hasPartido(siglas[1]) --Resgata o partido caso ja tenha sido criado.

  if partido == nil then--Cria um novo caso não encontre.
    local siglaColigacao = ""
    partido = Partido:new(siglas[1])

    -------Adquiri lista de coligações
    if siglas[2] ~= nil then
      siglaColigacao = siglas[2]:sub(2, #siglas[2])--(sub é apenas para )
    else--Caso o partido nao tenha uma coligacao, a sua prória sigla será a sigla de sua coligação.
      siglaColigacao = siglas[1]
    end

    local coligacao = self:hasColigacao(siglaColigacao)--Resgata a coligação caso ja tenha sido criada.
    if coligacao == nil then--Cria uma nova caso não encontre.
      coligacao = Coligacao:new(siglaColigacao)
      self:addColigacao(coligacao)
    end

    partido.coligacao = coligacao

    table.insert(self.partidos, partido)
  end

  if Candidato.eleito then
    partido:addCandidatosEleitos()
    partido.coligacao:addCandidatosEleitos()
  end

  Candidato.partido = partido
end

function Urna:addTotalVotos(votos)
  self.totalVotos = self.totalVotos + votos
end

function Urna:addNumVagas()
  self.numeroVagas = self.numeroVagas + 1
end

function remover(id, lista)
  for i,x in pairs(lista) do
    if x.id == id then
      table.remove(lista, i)
    end
  end
end

function Urna:addColigacao(coligacao)
  table.insert(self.coligacoes, coligacao)
end

function Urna:addCandidato(candidato)
  table.insert(self.candidatos, candidato)
end

function Urna:removeCandidato(candidato)
  remover(candidato.numero, self.candidatos)
end

function Urna:sortCandidatos()
  table.sort(self.candidatos, comparador_candidatos)
end

function Urna:sortPartidos()
  table.sort(self.partidos, comparador_partidos)
end

function Urna:sortColigacoes()
  table.sort(self.coligacoes, comparador_coligacoes)
end

function Urna:printPartidos()
  for i,x in pairs(self.partidos) do
    if x.candidatosEleitos > 1 then
      print(string.format("%i - %s, %i votos, %i candidatos eleitos", i, x.sigla, x.votos, x.candidatosEleitos))
    else
      print(string.format("%i - %s, %i votos, %i candidato eleito", i, x.sigla, x.votos, x.candidatosEleitos))
    end
  end
end

function Urna:printCandidatosMaisVotados()
  for i,x in pairs(self.candidatos) do
    if x.partido.coligacao == nil then
      print(string.format("%i - %s (%s, %i votos)", i, x.nome, x.partido.sigla, x.votos))
    else
      print(string.format("%i - %s (%s, %i votos) - Coligação: %s", i, x.nome, x.partido.sigla, x.votos, x.partido.coligacao.partidos))
    end
    if i == self.numeroVagas then break end
  end
end

function Urna:printCandidatosEleitos()
  local j = 1

  for i,x in pairs(self.candidatos) do
    if x.eleito == true then
      if x.partido.coligacao == nil then
        print(string.format("%i - %s (%s, %i votos)", j, x.nome, x.partido.sigla, x.votos))
      else
        print(string.format("%i - %s (%s, %i votos) - Coligação: %s", j, x.nome, x.partido.sigla, x.votos, x.partido.coligacao.partidos))
      end
      if j == self.numeroVagas then break end
      j = j + 1
    end
  end
end

function Urna:printNaoEleitosComMaisVotos()
  for i,x in pairs(self.candidatos) do
    if x.eleito == false then
      if x.partido.coligacao == nil then
        print(string.format("%i - %s (%s, %i votos)", i, x.nome, x.partido.sigla, x.votos))
      else
        print(string.format("%i - %s (%s, %i votos) - Coligação: %s", i, x.nome, x.partido.sigla, x.votos, x.partido.coligacao.partidos))
      end
    end
    if i == self.numeroVagas then break end
  end
end

function Urna:printBeneficiadosDoSistema()
  for i,x in pairs(self.candidatos) do
    if i > self.numeroVagas then
      if x.eleito == true then
        if x.partido.coligacao == nil then
          print(string.format("%i - %s (%s, %i votos)", i, x.nome, x.partido.sigla, x.votos))
        else
          print(string.format("%i - %s (%s, %i votos) - Coligação: %s", i, x.nome, x.partido.sigla, x.votos, x.partido.coligacao.partidos))
        end
      end
    end
  end
end

function Urna:printColigacoes()
  for i,x in pairs(self.coligacoes) do
    if x.candidatosEleitos > 1 then
      print(string.format("Coligação: %s, %i votos, %i candidatos eleitos", x.partidos, x.totalVotos, x.candidatosEleitos))
    else
      print(string.format("Coligação: %s, %i votos, %i candidato eleito", x.partidos, x.totalVotos, x.candidatosEleitos))
    end
  end
end

return Urna
