require("Leitura")
require("Utilitario")
local Candidato = require("Candidato")
local Urna = require("Urna")

local urna = Urna:new()
local linhas = linhas_from(arg[1])

if linhas == nil then
  print("Arquivo não encontrado!")
else
  local i = 2

  while i <= #linhas do
    if string.find(linhas[i], ";") == nil then break end --Se ler uma linha invalida, quebra a iteração.

    local colunas = split(linhas[i], ";")
    local candidato = Candidato:new()

    ----Leitura das colunas:
    --Coluna 1 -> Verifica se o candidato foi eleito.
    if string.find(colunas[1], "*") ~= nil then
      candidato.eleito = true
      urna:addNumVagas()
    end

    --Coluna 2 -> Registra o numero do candidato.
    candidato.numero = (tonumber(colunas[2]))

    --Coluna 3 -> Registra o nome do candidato.
    candidato.nome = colunas[3]

    --Coluna 4 -> Inicializa o Partido e o atribui ao candidato.
    urna:addPartido(colunas[4], candidato)

    --Coluna 5 -> Registra a quantidade de votos.
    local str = string.gsub(colunas[5], "%.", "") --Exclui o ponto para evitar ler o numero como um float.
    candidato:setVotos(tonumber(str))
    urna:addTotalVotos(tonumber(str))
    if candidato.partido.coligacao ~= nil then
      candidato.partido.coligacao:addTotalVotos(tonumber(str))
    end

    --Coluna 6 -> Registra a porcentagem dos votos validos.
    str = string.gsub(string.gsub(colunas[6], " %%", ""), "%,", ".")
    candidato.porcentagemVotos = (tonumber(str))

    ------FIM DA LEITURA DAS COLUNAS.

    urna:addCandidato(candidato)

    i = i + 1
  end

  urna:sortCandidatos()
  urna:sortPartidos()
  urna:sortColigacoes()

  --Prints no terminal
  print(string.format("Número de vagas: %i", urna.numeroVagas))

  print(string.format("\nVereadores eleitos:"))
  urna:printCandidatosEleitos();

  print(string.format("\nCandidatos mais votados (em ordem decrescente de votação e respeitando número de vagas):"))
  urna:printCandidatosMaisVotados();

  print(string.format("\nTeriam sido eleitos se a votação fosse majoritária, e não foram eleitos:\n(com sua posição no ranking de mais votados)"))
  urna:printNaoEleitosComMaisVotos()

  print(string.format("\nEleitos, que se beneficiaram do sistema proporcional:\n(com sua posição no ranking de mais votados)"))
  urna:printBeneficiadosDoSistema()

  print(string.format("\nVotação (nominal) das coligações e número de candidatos eleitos:"))
  urna:printColigacoes()

  print(string.format("\nVotação (nominal) dos partidos e número de candidatos eleitos:"))
  urna:printPartidos()

  print(string.format("\nTotal de votos nominais: %i", urna.totalVotos))
end
